#!/usr/bin/env python
# -*- encoding: utf-8 -*-

"""
@File        :   Phillip
@Time        :   2023/9/14 15:40
@Author      :   Xuesong Chen
@Description :
"""

from wuji.Reader.utils import get_equal_duration_and_labeled_chunks
from wuji.Reader.Annotation.Base import Base, AHI, AHEventFilter, merge_drop_duplicates_sort
import pandas as pd
import xmltodict
from datetime import datetime


class PhilipsAnnotationReader(Base):
    def __init__(self, file_path):
        super().__init__(file_path)

    def _parse_file(self, file_path):
        f = open(file_path, encoding='utf-8')
        self.info_dict = xmltodict.parse(f.read())
        start_time_str = self.info_dict['PatientStudy']['Acquisition']['Sessions']['Session'][
            'RecordingStart']
        self.recording_start_time = datetime.strptime(start_time_str, '%Y-%m-%dT%H:%M:%S')
        self.duration = int(self.info_dict['PatientStudy']['Acquisition']['Sessions']['Session']['Duration'])

    def get_standard_sleep_stages(self, drop_not_scored=False, **kwargs):
        stages = self.get_sleep_stages()
        self.sleep_stages = get_equal_duration_and_labeled_chunks(stages, **kwargs)
        if drop_not_scored:
            self.sleep_stages = self.sleep_stages[self.sleep_stages['Type'].isin(['Wake', 'N1', 'N2', 'N3', 'REM'])]
        return self.sleep_stages

    def get_sleep_stages(self):
        stage_list = self.info_dict['PatientStudy']['ScoringData']['StagingData'][
            'UserStaging']['NeuroAdultAASMStaging']['Stage']
        stage_dic = {
            'Start': [int(i['@Start']) for i in stage_list],
            'Type': [i['@Type'] for i in stage_list],
        }
        stages = pd.DataFrame.from_dict(stage_dic)
        stages['Duration'] = stages['Start'].shift(-1) - stages['Start']
        stages.at[stages.index[-1], 'Duration'] = self.duration - stages['Start'].iloc[-1]
        map_dic = {
            'Wake': 'Wake', 'NonREM1': 'N1', 'NonREM2': 'N2',
            'NonREM3': 'N3', 'NonREM4': 'N3', 'REM': 'REM',
            'NotScored': 'NotScored'
        }
        stages.loc[:, 'Type'] = stages['Type'].map(map_dic)
        return stages

    def get_standard_AH_events(self, type='AHI3', od_eps=45, aro_eps=6):

        if type == 'AHI4':
            self.get_OD_events(OD_level=4)
            self.arousal_events = None
        elif type == 'AHI3':
            self.get_OD_events(OD_level=3)
            self.get_arousal_events()

        self.get_standard_AH_events_H_with_machine()
        self.get_standard_AH_events_H_with_no_machine()

        filter = AHEventFilter(self.respiratory_events_H_with_machine, self.OD_events, self.arousal_events)
        res_H_with_machine = filter.get_filtered_AH_events(type=type, od_eps=od_eps, aro_eps=aro_eps)
        return merge_drop_duplicates_sort(res_H_with_machine, self.respiratory_events_H_with_no_machine)

    def get_respiratory_events(self):
        events_list = self.info_dict['PatientStudy']['ScoringData']['Events']['Event']
        resp_events_list = [i for i in events_list if i['@Family'] == 'Respiratory']
        type_list = []
        start_list = []
        duration_list = []
        map_dic = {
            'MixedApnea': 'Apnea',
            'CentralApnea': 'Apnea',
            'ObstructiveApnea': 'Apnea',
            'Hypopnea': 'Hypopnea',
        }
        for e in resp_events_list:
            if 10 <= float(e['@Duration']) <= 120:
                if e['@Type'] in ['PeriodicRespiration']:
                    continue
                type_list.append(map_dic[e['@Type']])
                start_list.append(float(e['@Start']))
                duration_list.append(float(e['@Duration']))
        respiratory_events_dic = {
            'Type': type_list,
            'Start': start_list,
            'Duration': duration_list,
        }
        self.respiratory_events = pd.DataFrame.from_dict(respiratory_events_dic)
        return self.respiratory_events

    # 返回 10 < float(e['@Duration']) < 120 的 所有Apnea, 和 只有'Machine' == 'true 的Hypopnea
    def get_standard_AH_events_H_with_machine(self):
        events_list = self.info_dict['PatientStudy']['ScoringData']['Events']['Event']
        resp_events_list = [i for i in events_list if i['@Family'] == 'Respiratory']

        type_list = []
        start_list = []
        duration_list = []
        map_dic = {
            'MixedApnea': 'Apnea',
            'CentralApnea': 'Apnea',
            'ObstructiveApnea': 'Apnea',
            'Hypopnea': 'Hypopnea',
        }

        for e in resp_events_list:
            if 10 <= float(e['@Duration']) <= 120:
                if e['@Type'] in ['PeriodicRespiration']:
                    continue
                if e['@Type'] == 'Hypopnea' and e.get('@Machine') == 'true':
                    type_list.append(map_dic[e['@Type']])
                    start_list.append(float(e['@Start']))
                    duration_list.append(float(e['@Duration']))
                elif e['@Type'] in ['MixedApnea', 'CentralApnea', 'ObstructiveApnea']:
                    type_list.append(map_dic[e['@Type']])
                    start_list.append(float(e['@Start']))
                    duration_list.append(float(e['@Duration']))

        respiratory_events_dic = {
            'Type': type_list,
            'Start': start_list,
            'Duration': duration_list,
        }

        self.respiratory_events_H_with_machine = pd.DataFrame.from_dict(respiratory_events_dic)
        return self.respiratory_events_H_with_machine

    # 只返回 是Hypopnea 且 不是机器标注的数据
    def get_standard_AH_events_H_with_no_machine(self):
        events_list = self.info_dict['PatientStudy']['ScoringData']['Events']['Event']
        resp_events_list = [i for i in events_list if i['@Family'] == 'Respiratory']

        type_list = []
        start_list = []
        duration_list = []
        map_dic = {
            'MixedApnea': 'Apnea',
            'CentralApnea': 'Apnea',
            'ObstructiveApnea': 'Apnea',
            'Hypopnea': 'Hypopnea',
        }

        for e in resp_events_list:
            if 10 <= float(e['@Duration']) <= 120:
                if e['@Type'] == 'Hypopnea' and e.get('@Machine') != 'true':
                    type_list.append(map_dic[e['@Type']])
                    start_list.append(float(e['@Start']))
                    duration_list.append(float(e['@Duration']))

        respiratory_events_dic = {
            'Type': type_list,
            'Start': start_list,
            'Duration': duration_list,
        }

        self.respiratory_events_H_with_no_machine = pd.DataFrame.from_dict(respiratory_events_dic)
        return self.respiratory_events_H_with_no_machine

    def get_OD_events(self, OD_level=3):  # 3 for 1A and 4 for 1B
        events_list = self.info_dict['PatientStudy']['ScoringData']['Events']['Event']
        OD_events_list = [i for i in events_list if i['@Family'] == 'SpO2']
        type_list = []
        start_list = []
        duration_list = []
        map_dic = {
            'RelativeDesaturation': 'Desaturation',
            'AbsoluteDesaturation': 'Desaturation',
        }
        for e in OD_events_list:
            ODBefore = int(e.get('O2Before', 0))
            ODMin = int(e.get('O2Min', 0))

            # Add the condition: ODBefore minus ODMin should be greater than or equal to 4
            if ODBefore - ODMin >= OD_level:
                type_list.append(map_dic[e['@Type']])
                start_list.append(float(e['@Start']))
                duration_list.append(float(e['@Duration']))

        OD_events_dic = {
            'Type': type_list,
            'Start': start_list,
            'Duration': duration_list,
        }
        OD_events = pd.DataFrame.from_dict(OD_events_dic)
        self.OD_events = OD_events
        return self.OD_events

    def get_arousal_events(self):
        events_list = self.info_dict['PatientStudy']['ScoringData']['Events']['Event']
        arousal_events_list = [i for i in events_list if i['@Family'] == 'Neuro']
        # print('arousal_events_list', arousal_events_list)
        type_list = []
        start_list = []
        duration_list = []
        map_dic = {
            'Arousal': 'Arousal',
        }

        for e in arousal_events_list:
            event_type = e['@Type']

            # Skip events with type 'REMSleepBehaviorDisorder'
            if event_type == 'REMSleepBehaviorDisorder':
                continue

            # Only include events with type 'Arousal'
            if event_type == 'Arousal':
                type_list.append(map_dic[event_type])
                start_list.append(float(e['@Start']))
                duration_list.append(float(e['@Duration']))

        arousal_events_dic = {
            'Type': type_list,
            'Start': start_list,
            'Duration': duration_list,
        }
        arousal_events = pd.DataFrame.from_dict(arousal_events_dic)
        self.arousal_events = arousal_events
        return self.arousal_events


def test_AHI(sleep_stages, respiratory_events):
    ahi_instance = AHI(respiratory_events, sleep_stages)

    total_ahi = ahi_instance.get_AHI(type='Total')
    rem_ahi = ahi_instance.get_AHI(type='REM')
    nrem_ahi = ahi_instance.get_AHI(type='NREM')

    return total_ahi, rem_ahi, nrem_ahi


if __name__ == '__main__':
    fp = '/Users/cxs/Downloads/00000753-LEBS21876_3211321.rml'
    anno = PhilipsAnnotationReader(fp)
    sleep_stages = anno.get_standard_sleep_stages()
    AH_events = anno.get_standard_AH_events()
    ahi = AHI(AH_events, sleep_stages)
    print(ahi.get_AHI(type='Total'))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Metrics
-------

Main functions:

* :func:`sed_eval.audio_tag.AudioTaggingMetrics.evaluate`: Calculate intermediate values for evaluation and accumulate them.
* :func:`sed_eval.audio_tag.AudioTaggingMetrics.results`: Calculate and return all metrics.
* :func:`sed_eval.audio_tag.AudioTaggingMetrics.results_overall_metrics`: Calculate and return overall metrics (micro-averaged).
* :func:`sed_eval.audio_tag.AudioTaggingMetrics.results_class_wise_metrics`: Calculate and return class-wise metrics.
* :func:`sed_eval.audio_tag.AudioTaggingMetrics.results_class_wise_average_metrics`: Calculate and return class-wise average metrics (macro-averaged).

Function :func:`sed_eval.audio_tag.AudioTaggingMetrics.evaluate` takes as a parameter tag lists,
(use :func:`dcase_util.containers.MetaDataContainer` to read them from a file), and probability lists (use :func:`dcase_util.containers.ProbabilityContainer` to read them from a file).

Usage example:

.. code-block:: python
    :linenos:

    import sed_eval
    import dcase_util

    reference_tag_list = dcase_util.containers.MetaDataContainer([
        {
            'filename': 'test1.wav',
            'tags': 'cat,dog'
        },
        {
            'filename': 'test2.wav',
            'tags': 'dog'
        },
        {
            'filename': 'test3.wav',
            'tags': 'bird,cat'
        },
        {
            'filename': 'test4.wav',
            'tags': 'cat'
        },
        {
            'filename': 'test5.wav',
            'tags': 'bird,speech'
        },
        {
            'filename': 'test6.wav',
            'tags': 'dog,speech'
        },
        {
            'filename': 'test7.wav',
            'tags': 'speech'
        },
    ])

    estimated_tag_probabilities = dcase_util.containers.ProbabilityContainer([
        {
            'filename': 'test1.wav',
            'label': 'bird',
            'probability': 0.2
        },
        {
            'filename': 'test1.wav',
            'label': 'cat',
            'probability': 0.99
        },
        {
            'filename': 'test1.wav',
            'label': 'dog',
            'probability': 0.88
        },
        {
            'filename': 'test1.wav',
            'label': 'speech',
            'probability': 0.01
        },

        {
            'filename': 'test2.wav',
            'label': 'bird',
            'probability': 0.1
        },
        {
            'filename': 'test2.wav',
            'label': 'cat',
            'probability': 0.3
        },
        {
            'filename': 'test2.wav',
            'label': 'dog',
            'probability': 0.8
        },
        {
            'filename': 'test2.wav',
            'label': 'speech',
            'probability': 0.1
        },


        {
            'filename': 'test3.wav',
            'label': 'bird',
            'probability': 0.7
        },
        {
            'filename': 'test3.wav',
            'label': 'cat',
            'probability': 0.6
        },
        {
            'filename': 'test3.wav',
            'label': 'dog',
            'probability': 0.4
        },
        {
            'filename': 'test3.wav',
            'label': 'speech',
            'probability': 0.3
        },

        {
            'filename': 'test4.wav',
            'label': 'bird',
            'probability': 0.323
        },
        {
            'filename': 'test4.wav',
            'label': 'cat',
            'probability': 0.6
        },
        {
            'filename': 'test4.wav',
            'label': 'dog',
            'probability': 0.56
        },
        {
            'filename': 'test4.wav',
            'label': 'speech',
            'probability': 0.4
        },


        {
            'filename': 'test5.wav',
            'label': 'bird',
            'probability': 0.8
        },
        {
            'filename': 'test5.wav',
            'label': 'cat',
            'probability': 0.7
        },
        {
            'filename': 'test5.wav',
            'label': 'dog',
            'probability': 0.45
        },
        {
            'filename': 'test5.wav',
            'label': 'speech',
            'probability': 0.43
        },


        {
            'filename': 'test6.wav',
            'label': 'bird',
            'probability': 0.9
        },
        {
            'filename': 'test6.wav',
            'label': 'cat',
            'probability': 0.53
        },
        {
            'filename': 'test6.wav',
            'label': 'dog',
            'probability': 0.83
        },
        {
            'filename': 'test6.wav',
            'label': 'speech',
            'probability': 0.95
        },


        {
            'filename': 'test7.wav',
            'label': 'bird',
            'probability': 0.2
        },
        {
            'filename': 'test7.wav',
            'label': 'cat',
            'probability': 0.2
        },
        {
            'filename': 'test7.wav',
            'label': 'dog',
            'probability': 0.89
        },
        {
            'filename': 'test7.wav',
            'label': 'speech',
            'probability': 0.45
        },
    ])

    estimated_tag_list = dcase_util.containers.MetaDataContainer()
    for file in estimated_tag_probabilities.unique_files:
        k = estimated_tag_probabilities.filter(filename=file)
        tags = []
        for item in k:
            if item.probability > 0.5:
                tags.append(item.label)

        estimated_tag_list.append(
            {
                'filename': file,
                'tags': tags
            }
        )

    tag_evaluator = sed_eval.audio_tag.AudioTaggingMetrics(
        tags=reference_tag_list.unique_tags
    )

    tag_evaluator.evaluate(
        reference_tag_list=reference_tag_list,
        estimated_tag_list=estimated_tag_list,
        estimated_tag_probabilities=estimated_tag_probabilities
    )
    print(tag_evaluator)


.. autosummary::
    :toctree: generated/

    AudioTaggingMetrics
    AudioTaggingMetrics.evaluate
    AudioTaggingMetrics.results
    AudioTaggingMetrics.results_overall_metrics
    AudioTaggingMetrics.results_class_wise_metrics
    AudioTaggingMetrics.results_class_wise_average_metrics
    AudioTaggingMetrics.result_report_parameters
    AudioTaggingMetrics.result_report_class_wise
    AudioTaggingMetrics.result_report_class_wise_average
    AudioTaggingMetrics.reset

"""

from __future__ import absolute_import
import numpy
from . import metric
import dcase_util


class AudioTaggingMetrics:
    def __init__(self, tags=None):
        self.tag_label_list = tags

        self.overall = {
            'Ncorr': 0.0,
            'Nref': 0.0,
            'Nsys': 0.0,
            'Ntp': 0.0,
            'Ntn': 0.0,
            'Nfp': 0.0,
            'Nfn': 0.0,
        }

        self.tag_wise = {}
        self.y_true = {}
        self.y_pred = {}
        self.y_pred_score = {}

        for label in self.tag_label_list:
            self.tag_wise[label] = {
                'Nref': 0.0,
                'Nsys': 0.0,
                'Ntp': 0.0,
                'Ntn': 0.0,
                'Nfp': 0.0,
                'Nfn': 0.0,
            }
            self.y_true[label] = []
            self.y_pred[label] = []
            self.y_pred_score[label] = []

        self.ui = dcase_util.ui.FancyStringifier()

    def __str__(self):
        """Print result reports"""

        output = self.ui.section_header('Audio tagging metrics') + '\n'
        output += self.result_report_parameters() + '\n'
        output += self.result_report_overall() + '\n'
        output += self.result_report_class_wise_average() + '\n'
        output += self.result_report_class_wise() + '\n'

        return output

    def evaluate(self, reference_tag_list, estimated_tag_list=None, estimated_tag_probabilities=None):
        """Evaluate estimated against reference

        Parameters
        ----------

        reference_tag_list : list of dict or dcase_util.containers.MetaDataContainer
            Reference tag list

        estimated_tag_list : list of dict or dcase_util.containers.MetaDataContainer
            Estimated tag list

        estimated_tag_probabilities : list of dict or dcase_util.containers.ProbabilityContainer
            Estimated tag probabilities

        Returns
        -------
        self

        """

        if estimated_tag_list is None and estimated_tag_probabilities is None:
            raise ValueError("Nothing to evaluate, give at least estimated_tag_list or estimated_tag_probabilities")

        # Make sure reference_tag_list is dcase_util.containers.MetaDataContainer
        if not isinstance(reference_tag_list, dcase_util.containers.MetaDataContainer):
            reference_tag_list = dcase_util.containers.MetaDataContainer(reference_tag_list)

        # Make sure estimated_tag_list is dcase_util.containers.MetaDataContainer
        if estimated_tag_list is not None:
            if not isinstance(estimated_tag_list, dcase_util.containers.MetaDataContainer):
                estimated_tag_list = dcase_util.containers.MetaDataContainer(estimated_tag_list)

        # Make sure estimated_tag_probabilities is dcase_util.containers.ProbabilityContainer
        if estimated_tag_probabilities is not None:
            if not isinstance(estimated_tag_probabilities, dcase_util.containers.ProbabilityContainer):
                estimated_tag_probabilities = dcase_util.containers.ProbabilityContainer(estimated_tag_probabilities)

        y_true = []
        y_pred = []

        # Go though reference and estimated list label by label, and file by file
        for label in self.tag_label_list:
            for filename in reference_tag_list.unique_files:
                reference_item = reference_tag_list.filter(filename=filename)[0]

                # Populate y_true based on reference_item
                if label in reference_item.tags:
                    self.y_true[label].append(1)
                    y_true.append(1)

                else:
                    self.y_true[label].append(0)
                    y_true.append(0)

                if estimated_tag_list is not None:
                    # Evaluate based on estimated tags

                    estimated_item = estimated_tag_list.filter(filename=filename)[0]

                    if not estimated_item:
                        raise ValueError(
                            "Not all reference files estimated, please check [{file}]".format(
                                file=filename
                            )
                        )
                    # Store nref
                    if label in reference_item.tags:
                        self.tag_wise[label]['Nref'] += 1

                    # Populate y_pred based estimated_item
                    if label in estimated_item.tags:
                        self.y_pred[label].append(1)
                        y_pred.append(1)
                        self.tag_wise[label]['Nsys'] += 1

                    else:
                        self.y_pred[label].append(0)
                        y_pred.append(0)

                    # Accumulate intermediate values
                    # True positives (TP)
                    if label in reference_item.tags and label in estimated_item.tags:
                        self.tag_wise[label]['Ntp'] += 1

                    # True negatives (TN)
                    if label not in reference_item.tags and label not in estimated_item.tags:
                        self.tag_wise[label]['Ntn'] += 1

                    # False positives (FP)
                    if label not in reference_item.tags and label in estimated_item.tags:
                        self.tag_wise[label]['Nfp'] += 1

                    # False negatives (FN)
                    if label in reference_item.tags and label not in estimated_item.tags:
                        self.tag_wise[label]['Nfn'] += 1

                if estimated_tag_probabilities is not None:
                    # Evaluate based on per tag probabilities

                    estimated_item = estimated_tag_probabilities.filter(filename=filename, label=label)[0]
                    self.y_pred_score[label].append(float(estimated_item['probability']))

        if estimated_tag_list is not None:
            # Evaluate based on estimated tags

            self.overall['Nref'] += sum(y_true)
            self.overall['Nsys'] += sum(y_pred)

            y_true = numpy.array(y_true)
            y_pred = numpy.array(y_pred)

            self.overall['Ntp'] += sum(y_pred + y_true > 1)
            self.overall['Ntn'] += sum(y_pred + y_true == 0)
            self.overall['Nfp'] += sum(y_pred - y_true > 0)
            self.overall['Nfn'] += sum(y_true - y_pred > 0)

        return self

    def reset(self):
        """Reset internal state
        """
        self.overall = {
            'Nref': 0.0,
            'Nsys': 0.0,
            'Ntp': 0.0,
            'Ntn': 0.0,
            'Nfp': 0.0,
            'Nfn': 0.0,
        }

        self.tag_wise = {}
        for label in self.tag_label_list:
            self.tag_wise[label] = {
                'Nref': 0.0,
                'Nsys': 0.0,
                'Ntp': 0.0,
                'Ntn': 0.0,
                'Nfp': 0.0,
                'Nfn': 0.0,
            }

        return self

    # Results
    def results(self):
        """All metrics

        Returns
        -------
        dict
            results in a dictionary format

        """

        results = {
            'overall': self.results_overall_metrics(),
            'class_wise': self.results_class_wise_metrics(),
            'class_wise_average': self.results_class_wise_average_metrics(),
        }

        return results

    def results_overall_metrics(self):
        """Overall metrics

        Returns
        -------
        dict
            results in a dictionary format

        """

        # F-measure
        if self.overall['Nref'] > 0:
            precision = metric.precision(
                Ntp=self.overall['Ntp'],
                Nsys=self.overall['Nsys']
            )

            recall = metric.recall(
                Ntp=self.overall['Ntp'],
                Nref=self.overall['Nref']
            )

            f_measure = metric.f_measure(
                precision=precision,
                recall=recall
            )

        else:
            precision = None
            recall = None
            f_measure = None

        y_true = []
        y_score = []
        for tag_id, tag_label in enumerate(self.tag_label_list):
            y_true += self.y_true[tag_label]
            y_score += self.y_pred_score[tag_label]

        if y_score:
            eer = metric.equal_error_rate(
                y_true=y_true,
                y_score=y_score
            )

        else:
            eer = None

        return {
                'count': {
                    'Nref': self.overall['Nref'],
                    'Nsys': self.overall['Nsys'],
                },
                'f_measure': {
                    'f_measure': f_measure,
                    'precision': precision,
                    'recall': recall,
                },
                'eer': {
                    'eer': eer
                }

        }

    def results_class_wise_metrics(self):
        """Class-wise metrics

        Returns
        -------
        dict
            results in a dictionary format

        """

        results = {}
        for tag_id, tag_label in enumerate(self.tag_label_list):
            if tag_label not in results:
                results[tag_label] = {}

            # Counts
            results[tag_label]['count'] = {
                'Nref': self.tag_wise[tag_label]['Nref'],
                'Nsys': self.tag_wise[tag_label]['Nsys'],
                'Ntp': self.tag_wise[tag_label]['Ntp'],
                'Ntn': self.tag_wise[tag_label]['Ntn'],
                'Nfp': self.tag_wise[tag_label]['Nfp'],
                'Nfn': self.tag_wise[tag_label]['Nfn'],
            }

            # Equal error rate
            if self.y_pred_score[tag_label]:
                results[tag_label]['eer'] = {
                    'eer': metric.equal_error_rate(
                        y_true=self.y_true[tag_label],
                        y_score=self.y_pred_score[tag_label]
                    )
                }

            else:
                results[tag_label]['eer'] = {
                    'eer': None
                }

            # F-measure
            if self.tag_wise[tag_label]['Nref'] > 0:

                precision = metric.precision(
                    Ntp=self.tag_wise[tag_label]['Ntp'],
                    Nsys=self.tag_wise[tag_label]['Nsys']
                )

                recall = metric.recall(
                    Ntp=self.tag_wise[tag_label]['Ntp'],
                    Nref=self.tag_wise[tag_label]['Nref']
                )

                f_measure = metric.f_measure(precision=precision, recall=recall)

            else:
                precision = None
                recall = None
                f_measure = None

            results[tag_label]['f_measure'] = {
                'f_measure': f_measure,
                'precision': precision,
                'recall': recall,
            }

        return results

    def results_class_wise_average_metrics(self):
        """Class-wise averaged metrics

        Returns
        -------
        dict
            results in a dictionary format

        """

        class_wise_results = self.results_class_wise_metrics()

        class_wise_eer = []
        class_wise_fmeasure = []
        class_wise_precision = []
        class_wise_recall = []

        for class_label in class_wise_results:
            if class_wise_results[class_label]['eer']['eer'] is not None:
                class_wise_eer.append(class_wise_results[class_label]['eer']['eer'])

            if class_wise_results[class_label]['f_measure']['f_measure'] is not None:
                class_wise_fmeasure.append(class_wise_results[class_label]['f_measure']['f_measure'])
                class_wise_precision.append(class_wise_results[class_label]['f_measure']['precision'])
                class_wise_recall.append(class_wise_results[class_label]['f_measure']['recall'])

        if class_wise_eer:
            eer = float(numpy.nanmean(class_wise_eer))

        else:
            eer = None

        if class_wise_fmeasure:
            f_measure = float(numpy.nanmean(class_wise_fmeasure))

        else:
            f_measure = None

        if class_wise_precision:
            precision = float(numpy.nanmean(class_wise_precision))

        else:
            precision = None

        if class_wise_recall:
            recall = float(numpy.nanmean(class_wise_recall))

        else:
            recall = None

        return {
            'eer': {
                'eer': eer
            },
            'f_measure': {
                'f_measure': f_measure,
                'precision': precision,
                'recall': recall,
            }
        }

    # Reports
    def result_report_parameters(self):
        """Report metric parameters

        Returns
        -------
        str
            result report in string format

        """

        output = self.ui.data(field='Tags', value=len(self.tag_label_list)) + '\n'
        output += self.ui.data(field='Evaluated units', value=int(self.overall['Nref'])) + '\n'

        return output

    def result_report_overall(self):
        """Report overall results

        Returns
        -------
        str
            result report in string format

        """

        results = self.results_overall_metrics()

        output = self.ui.section_header('Overall metrics (micro-average)', indent=2) + '\n'

        if 'f_measure' in results and results['f_measure']:

            if results['f_measure']['f_measure'] is not None:
                f_measure = results['f_measure']['f_measure'] * 100

            else:
                f_measure = None

            if results['f_measure']['precision'] is not None:
                precision = results['f_measure']['precision'] * 100

            else:
                precision = None

            if results['f_measure']['recall'] is not None:
                recall = results['f_measure']['recall'] * 100

            else:
                recall = None

            output += self.ui.line('F-measure', indent=2) + '\n'
            output += self.ui.data(field='F-measure (F1)', value=f_measure, unit='%', indent=4) + '\n'
            output += self.ui.data(field='Precision', value=precision, unit='%', indent=4) + '\n'
            output += self.ui.data(field='Recall', value=recall, unit='%', indent=4) + '\n'

        if 'eer' in results and results['eer']:
            if results['eer']['eer'] is not None:
                eer = results['eer']['eer'] * 100

            else:
                eer = None

            output += self.ui.line('Equal error rate', indent=2) + '\n'
            output += self.ui.data(field='Equal error rate (EER)', value=eer, unit='%', indent=4) + '\n'

        return output

    def result_report_class_wise_average(self):
        """Report class-wise averages

        Returns
        -------
        str
            result report in string format

        """

        results = self.results_class_wise_average_metrics()

        output = self.ui.section_header('Class-wise average metrics (macro-average)', indent=2) + '\n'

        if 'f_measure' in results and results['f_measure']:
            if results['f_measure']['f_measure'] is not None:
                f_measure = results['f_measure']['f_measure'] * 100

            else:
                f_measure = None

            if results['f_measure']['precision'] is not None:
                precision = results['f_measure']['precision'] * 100

            else:
                precision = None

            if results['f_measure']['recall'] is not None:
                recall = results['f_measure']['recall'] * 100

            else:
                recall = None

            output += self.ui.line('F-measure', indent=2) + '\n'
            output += self.ui.data(field='F-measure (F1)', value=f_measure, unit='%', indent=4) + '\n'
            output += self.ui.data(field='Precision', value=precision, unit='%', indent=4) + '\n'
            output += self.ui.data(field='Recall', value=recall, unit='%', indent=4) + '\n'

        if 'eer' in results and results['eer']:
            if results['eer']['eer'] is not None:
                eer = results['eer']['eer'] * 100

            else:
                eer = None

            output += self.ui.line('Equal error rate', indent=2) + '\n'
            output += self.ui.data(field='Equal error rate (EER)', value=eer, unit='%', indent=4) + '\n'

        return output

    def result_report_class_wise(self):
        """Report class-wise results

        Returns
        -------
        str
            result report in string format

        """

        results = self.results_class_wise_metrics()

        output = self.ui.section_header('Class-wise metrics', indent=2) + '\n'
        output += self.ui.row(
            'Tag', 'Nref', 'Nsys', 'F-score', 'Pre', 'Rec', 'EER',
            widths=[20, 12, 12, 12, 12, 12, 12],
            separators=[True, False, True, False, False, True, False],
            indent=4
        ) + '\n'
        output += self.ui.row('-', '-', '-', '-', '-', '-', '-') + '\n'

        for tag_label in self.tag_label_list:
            if results[tag_label]['f_measure']['f_measure'] is not None:
                f_measure = results[tag_label]['f_measure']['f_measure'] * 100

            else:
                f_measure = None

            if results[tag_label]['f_measure']['precision'] is not None:
                precision = results[tag_label]['f_measure']['precision'] * 100

            else:
                precision = None

            if results[tag_label]['f_measure']['recall'] is not None:
                recall = results[tag_label]['f_measure']['recall'] * 100

            else:
                recall = None

            if results[tag_label]['eer']['eer'] is not None:
                eer = results[tag_label]['eer']['eer'] * 100

            else:
                eer = None

            output += self.ui.row(
                tag_label,
                results[tag_label]['count']['Nref'],
                results[tag_label]['count']['Nsys'],
                f_measure,
                precision,
                recall,
                eer,
                types=['str', 'int', 'int', 'float1_percentage', 'float1', 'float1', 'float1_percentage']
            ) + '\n'

        return output



#!/usr/bin/env python
# -*- encoding: utf-8 -*-

"""
@File        :   signal 
@Time        :   2023/11/21 14:06
@Author      :   Xuesong Chen
@Description :   
"""

import numpy as np


def interp_signal(positions, values, fs, target_fs, n_samples):
    '''
    Interpolate RRI or JJI to target sampling frequency
    Parameters
    ----------
    positions: positions of RRI or JJI
    values: values of RRI or JJI
    fs: sampling frequency of RRI or JJI
    target_fs: target sampling frequency
    n_samples: number of samples of the raw signal

    Returns
    -------
    interp_value: interpolated RRI or JJI at target sampling frequency, in ms
    '''
    interp_value = np.interp(
        np.arange(0, n_samples, fs // target_fs), positions,
        values / fs*1000).astype(np.float32)
    return interp_value



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Event matching
"""

def bipartite_match(graph):
    """
    Find maximum cardinality matching of a bipartite graph (U,V,E).
    Function is borrowed from mir_eval toolbox (https://github.com/craffel/mir_eval).

    The input format is a dictionary mapping members of U to a list of their neighbors in V.
    The output is a dict M mapping members of V to their matches in U.

    Parameters
    ----------
    graph : dictionary : left-vertex -> list of right vertices
        The input bipartite graph.  Each edge need only be specified once.

    Returns
    -------
    matching : dictionary : right-vertex -> left vertex
        A maximal bipartite matching.

    """
    # Implementation is after _bipartite_match function in mir_eval toolbox:
    # Colin Raffel, Brian McFee, Eric J. Humphrey, Justin Salamon, Oriol Nieto, Dawen Liang,
    # and Daniel P. W. Ellis, "mir_eval: A Transparent Implementation of Common MIR Metrics",
    # Proceedings of the 15th International Conference on Music Information Retrieval, 2014.
    #
    # _bipartite_match function:
    # https://github.com/craffel/mir_eval/blob/master/mir_eval/util.py#L547
    #
    # Function is originally adapted from:
    #
    # Hopcroft-Karp bipartite max-cardinality matching and max independent set
    # David Eppstein, UC Irvine, 27 Apr 2002

    # initialize greedy matching (redundant, but faster than full search)
    matching = {}
    for u in graph:
        for v in graph[u]:
            if v not in matching:
                matching[v] = u
                break

    while True:
        # structure residual graph into layers
        # pred[u] gives the neighbor in the previous layer for u in U
        # preds[v] gives a list of neighbors in the previous layer for v in V
        # unmatched gives a list of unmatched vertices in final layer of V,
        # and is also used as a flag value for pred[u] when u is in the first
        # layer

        preds = {}
        unmatched = []
        pred = dict([(u, unmatched) for u in graph])

        for v in matching:
            del pred[matching[v]]

        layer = list(pred)

        # repeatedly extend layering structure by another pair of layers
        while layer and not unmatched:
            new_layer = {}
            for u in layer:
                for v in graph[u]:
                    if v not in preds:
                        new_layer.setdefault(v, []).append(u)

            layer = []
            for v in new_layer:
                preds[v] = new_layer[v]

                if v in matching:
                    layer.append(matching[v])
                    pred[matching[v]] = v

                else:
                    unmatched.append(v)

        # did we finish layering without finding any alternating paths?
        if not unmatched:
            unlayered = {}
            for u in graph:
                for v in graph[u]:
                    if v not in preds:
                        unlayered[v] = None

            return matching

        def recurse(v):
            """Recursively search backward through layers to find alternating
            paths.  recursion returns true if found path, false otherwise
            """

            if v in preds:
                L = preds[v]
                del preds[v]
                for u in L:
                    if u in pred:
                        pu = pred[u]
                        del pred[u]
                        if pu is unmatched or recurse(pu):
                            matching[v] = u
                            return True

            return False

        for v in unmatched:
            recurse(v)


#!/usr/bin/env python
# -*- encoding: utf-8 -*-

"""
@File        :   AnnotationBase 
@Time        :   2023/8/17 16:21
@Author      :   Xuesong Chen
@Description :   
"""
from abc import ABC, abstractmethod
from wuji.Reader.utils import get_irequal_duration_chunks
import pandas as pd


def merge_drop_duplicates_sort(*args, sort_column='Start'):
    result_df = pd.concat(args)
    result_df = result_df.drop_duplicates()
    result_df = result_df.sort_values(by=sort_column)
    result_df = result_df.reset_index(drop=True)
    return result_df


class AHEventFilter:
    def __init__(self, respiratory_events, od_events, arousal_events=None):
        self.respiratory_events = respiratory_events
        self.od_events = od_events
        self.arousal_events = arousal_events

    def get_AH_with_OD(self, row, od_eps=45):
        if row['Type'] == 'Hypopnea':
            hypopnea_start = row['Start']
            hypopnea_end = hypopnea_start + row['Duration']

            # Check if this hypopnea event has already been processed
            if 'Processed_od' not in self.od_events.columns:
                self.od_events['Processed_od'] = False

            condition = ((hypopnea_start <= self.od_events['Start']) &
                         (hypopnea_end + od_eps > self.od_events['Start']) &
                         (~self.od_events['Processed_od']))
            rows_to_drop = self.od_events[condition].index

            if any(condition):
                # rows_to_drop可能有多个, [0] 将第一个匹配到的 OD 不再匹配后续其他的 H
                self.od_events.loc[rows_to_drop[0], 'Processed_od'] = True
                return True  # Break out of the loop if a match is found
            return False

        else:
            return True

    def get_AH_with_Arousal(self, row, aro_eps=6):
        if row['Type'] == 'Hypopnea':
            hypopnea_start = row['Start']
            hypopnea_end = hypopnea_start + row['Duration']

            if 'Processed_arousal' not in self.arousal_events.columns:
                self.arousal_events['Processed_arousal'] = False

            condition = ((hypopnea_start <= self.arousal_events['Start']) &
                         (hypopnea_end + aro_eps > self.arousal_events['Start']) &
                         (hypopnea_end < (self.arousal_events['Start'] + self.arousal_events['Duration'])) &
                         (~self.arousal_events['Processed_arousal']))

            rows_to_drop = self.arousal_events[condition].index

            if any(condition):
                # 第一个匹配到的 arousal 不再匹配后续其他的 H
                self.arousal_events.loc[rows_to_drop[0], 'Processed_arousal'] = True
                return True
            return False

        else:
            return True

    def get_filtered_AH_events(self, type='AHI3', od_eps=45, aro_eps=6):
        if type == 'AHI4':
            ah_with_od = self.respiratory_events[
                self.respiratory_events.apply(self.get_AH_with_OD, od_eps=od_eps, axis=1)]
            return ah_with_od
        elif type == 'AHI3':
            ah_with_od = self.respiratory_events[
                self.respiratory_events.apply(self.get_AH_with_OD, od_eps=od_eps, axis=1)]
            ah_with_arousal = self.respiratory_events[
                self.respiratory_events.apply(self.get_AH_with_Arousal, aro_eps=aro_eps, axis=1)]
            return merge_drop_duplicates_sort(ah_with_od, ah_with_arousal)


class AHI:
    def __init__(self, AH_events, sleep_stages):
        self.total_sleep_time = sleep_stages[~sleep_stages['Type'].isin(['Wake', 'NotScored'])]['Duration'].sum()
        self.total_sleep_time_in_hours = self.total_sleep_time / 3600
        self.total_sleep_time_in_hours_REM = sleep_stages[sleep_stages['Type'].isin(['REM'])]['Duration'].sum() / 3600
        self.total_sleep_time_in_hours_NREM = sleep_stages[sleep_stages['Type'].isin(['N1', 'N2', 'N3'])][
                                                  'Duration'].sum() / 3600
        self.sleep_stages = get_irequal_duration_chunks(sleep_stages)
        self.AH_events = AH_events
        self.sleep_stages['End'] = self.sleep_stages['Start'] + self.sleep_stages['Duration']
        self.AH_events['End'] = self.AH_events['Start'] + self.AH_events['Duration']
        self._assign_sleep_stage_label_to_respiratory_events()

    def _assign_sleep_stage_label_to_respiratory_events(self):
        if not self.AH_events.empty:
            for idx, row in self.sleep_stages.iterrows():
                start_time = row['Start']
                end_time = row['End']
                self.AH_events.loc[
                    (self.AH_events['Start'] >= start_time) & (self.AH_events['End'] <= end_time),
                    'SleepStage'] = row['Type']
            sleep_stage_idx_cache = 0
            for resp_idx, resp_row in self.AH_events[self.AH_events['SleepStage'].isna()].iterrows():
                for sleep_stage_idx, sleep_stage_row in self.sleep_stages.loc[sleep_stage_idx_cache:].iterrows():
                    if resp_row['End'] > sleep_stage_row['End'] and resp_row['Start'] < sleep_stage_row['End']:
                        overlap_on_cur_stage = sleep_stage_row['End'] - resp_row['Start']
                        overlap_on_next_stage = resp_row['End'] - sleep_stage_row['End']
                        cur_event_stage = None
                        stage_set = list(set([
                            sleep_stage_row['Type'],
                            self.sleep_stages.loc[sleep_stage_idx + 1, 'Type']
                        ]))
                        if len(stage_set) != 1 and 'Wake' in stage_set:
                            stage_set.remove('Wake')
                            cur_event_stage = stage_set[0]
                        if overlap_on_cur_stage >= overlap_on_next_stage:
                            cur_event_stage = sleep_stage_row['Type']
                        else:
                            cur_event_stage = self.sleep_stages.loc[sleep_stage_idx + 1, 'Type']
                        self.AH_events.loc[resp_idx, 'SleepStage'] = cur_event_stage
                        sleep_stage_idx_cache = sleep_stage_idx
                        break
            assert self.AH_events['SleepStage'].isna().sum() == 0, "存在未标记的呼吸事件"

    def get_AHI(self, type='Total'):

        if self.AH_events.empty:
            return 0

        if type == 'Total':
            ahi = self.AH_events['SleepStage'].isin(
                ['N1', 'N2', 'N3', 'REM']).sum() / self.total_sleep_time_in_hours
        elif type == 'REM':
            ahi = self.AH_events['SleepStage'].isin(['REM']).sum() / self.total_sleep_time_in_hours_REM
        elif type == 'NREM':
            ahi = self.AH_events['SleepStage'].isin(
                ['N1', 'N2', 'N3']).sum() / self.total_sleep_time_in_hours_NREM
        else:
            raise ValueError("Invalid type. Must be 'Total', 'REM', or 'NREM'.")

        return ahi

    def get_AH_events_in_sleep(self):
        return self.AH_events[self.AH_events['SleepStage'].isin(['N1', 'N2', 'N3', 'REM'])]


class Base(ABC):
    def __init__(self, file_path):
        self.sleep_stages = None
        self.respiratory_events = None
        self.AH_events = None
        self.OD_events = None
        self._parse_file(file_path)

    @abstractmethod
    def _parse_file(self, file_path):
        self.recording_start_time = None
        self.duration = None
        self.anno_df = None

    def get_recording_start_time(self):
        return self.recording_start_time

    def get_duration(self):
        return self.duration

    def get_sleep_onset_time(self):
        if self.sleep_stages is None:
            self.sleep_stages = self.get_standard_sleep_stages()
        onset_time = self.sleep_stages[self.sleep_stages['Type'].isin(['N1', 'N2', 'N3', 'REM'])]['Start'].values[0]
        return onset_time

    def total_sleep_time(self):
        if self.sleep_stages is None:
            self.sleep_stages = self.get_standard_sleep_stages()
        total_sleep_time = self.sleep_stages[self.sleep_stages['Type'].isin(['N1', 'N2', 'N3', 'REM'])][
            'Duration'].sum()
        return total_sleep_time

    def get_standard_sleep_stages(self):
        '''
        用于获取标准的睡眠分期标记

        | Type | Start | Duration |
        |------|-------|----------|

        Type: Wake, N1, N2, N3, REM
        Start: 从睡眠开始到当前分期的时间
        Duration: 当前分期的持续时间，统一为30s

        :return:
        上述Dataframe格式
        '''
        return None

    def get_standard_AH_events(self):
        '''
        用于获取标准的呼吸暂停、低通气标记

        | Type | Start | Duration |
        |------|-------|----------|

        Type: Apnea, Hypopnea
        Start: 从睡眠开始到当前事件的开始时间
        Duration: 当前事件的持续时间

        :return:
        上述Dataframe格式
        '''
        pass

    def plot_sleep_stage(self, ax=None):
        import matplotlib.pyplot as plt
        from matplotlib.dates import DateFormatter

        # 设置开始时间
        start_time = self.recording_start_time
        sleep_stage = self.get_standard_sleep_stages() if self.sleep_stages is None else self.sleep_stages

        # 按照指定的顺序更新类型映射
        type_order = ['N3', 'N2', 'N1', 'REM', 'Wake', 'NotScored']
        type_mapping = {type: i for i, type in enumerate(type_order)}
        # 创建颜色映射，根据每个类型的实际含义选择颜色
        color_mapping = {
            "N3": "darkblue",
            "N2": "purple",
            "N1": "blue",
            "REM": "green",
            "Wake": "lightblue",
            "NotScored": "grey"
        }

        sleep_stage['Start'] = pd.to_timedelta(sleep_stage['Start'], unit='s') + start_time
        sleep_stage['End'] = sleep_stage['Start'] + pd.to_timedelta(sleep_stage['Duration'], unit='s')

        # 创建图表
        if not ax:
            fig, ax = plt.subplots(figsize=(100, 5))

        prev_end = None
        prev_type = None
        for _, row in sleep_stage.iterrows():
            ax.barh(type_mapping[row['Type']], row['End'] - row['Start'], left=row['Start'], height=0.5,
                    align='center',
                    color=color_mapping[row['Type']])
            if prev_end is not None and prev_type is not None:
                ax.plot([prev_end, row['Start']], [type_mapping[prev_type], type_mapping[row['Type']]],
                        color='lightgrey')
            prev_end = row['End']
            prev_type = row['Type']

        # 设置y轴的标签
        ax.set_yticks(range(len(type_mapping)))
        ax.set_yticklabels(list(type_mapping.keys()))

        # 设置x轴的刻度标签为实际的小时和分钟
        ax.xaxis.set_major_formatter(DateFormatter('%H:%M'))
        # set x limits
        start_time = sleep_stage['Start'].values[0]
        end_time = sleep_stage['End'].values[-1]
        ax.set_xlim([start_time, end_time])



# !/usr/bin/env python
# -*- encoding: utf-8 -*-

"""
@File        :   Base 
@Time        :   2023/8/18 14:34
@Author      :   Xuesong Chen
@Description :   
"""
import re
import numpy as np
from pyedflib.edfreader import EdfReader


class Base:
    def __init__(self, file_path):
        self.reader = EdfReader(file_path)
        self.signal_labels = np.array(self.reader.getSignalLabels())
        self.duration = self.reader.getFileDuration()
        self._assign_signal_types()

    def _assign_signal_types(self):
        self.signal_type = []
        for idx, sig in enumerate(self.signal_labels):
            if re.search('E[CK]G', sig, re.IGNORECASE):
                self.signal_type.append('ecg')
            elif re.search('S[pa]O2', sig, re.IGNORECASE):
                self.signal_type.append('spo2')
            elif re.search('Effort ABD|ABDO', sig, re.IGNORECASE):
                self.signal_type.append('abd')
            elif re.search('CHEST|Effort THO', sig, re.IGNORECASE):
                self.signal_type.append('chest')
            elif re.search('EEG', sig, re.IGNORECASE):
                self.signal_type.append('eeg')
            elif re.search('EMG', sig, re.IGNORECASE):
                self.signal_type.append('emg')
            elif re.search('EOG', sig, re.IGNORECASE):
                self.signal_type.append('eog')
            elif re.search('Snore', sig, re.IGNORECASE):
                self.signal_type.append('snore')
            elif re.search('position', sig, re.IGNORECASE):
                self.signal_type.append('position')
            elif re.search('Flow|NEW AIR', sig, re.IGNORECASE):
                self.signal_type.append('flow')
            elif re.search('Numeric Aux', sig, re.IGNORECASE):
                self.signal_type.append('trigger')
            else:
                self.signal_type.append('unk')
        self.signal_type = np.array(self.signal_type)

    def get_start_recording_time(self):
        return self.reader.getStartdatetime()

    def get_number_of_signals(self, type='ecg'):
        return np.sum(self.signal_type == type)

    def get_signal(self, ch_name=None, type='ecg', tmin=None, tmax=None, order=0):
        '''
        获取所有匹配信号，order代表次序，0代表第一个匹配的信号
        :param ch_name:
        :param type:
        :param tmin:
        :param tmax:
        :param order:
        :return:
        '''
        if ch_name:
            idx = np.argwhere(self.signal_labels == ch_name).flatten()[order]
        else:
            idx = np.argwhere(self.signal_type == type).flatten()[order]
        sfreq = int(self.reader.getSampleFrequency(idx))
        if tmin is None or tmax is None:
            return self.reader.readSignal(idx)
        start_samp_idx = sfreq * tmin
        end_samp_idx = sfreq * tmax
        n_samples = end_samp_idx - start_samp_idx
        return self.reader.readSignal(idx, start_samp_idx, n=n_samples)

    def get_sample_frequency(self, ch_name=None, type='ecg', order=0):
        # 注：只获取第一个匹配的信号
        if ch_name:
            idx = np.argwhere(self.signal_labels == ch_name).flatten()[order]
        else:
            idx = np.argwhere(self.signal_type == type).flatten()[order]
        return int(self.reader.getSampleFrequency(idx))

    def get_duration(self):
        return self.reader.file_duration


if __name__ == '__main__':
    fp = '/Users/cxs/project/OSAPillow/data/SHHS/edfs/shhs1-200001.edf'
    reader = Base(fp)
    sig = reader.get_signal(type='ecg')
    print(reader.signal_labels)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Event roll handling

"""
from __future__ import absolute_import
import math
import numpy
from . import event_list
import dcase_util


def event_list_to_event_roll(source_event_list, event_label_list=None, time_resolution=0.01):
    """Convert event list into event roll, binary activity matrix

    Parameters
    ----------
    source_event_list : list, shape=(n,)
        A list containing event dicts

    event_label_list : list, shape=(k,) or None
        A list of containing unique labels in alphabetical order
        (Default value = None)
    time_resolution : float > 0
        Time resolution in seconds of the event roll
        (Default value = 0.01)

    Returns
    -------

    event_roll: np.ndarray, shape=(m,k)
        Event roll

    """

    if isinstance(source_event_list, dcase_util.containers.MetaDataContainer):
        max_offset_value = source_event_list.max_offset

        if event_label_list is None:
            event_label_list = source_event_list.unique_event_labels

    elif isinstance(source_event_list, list):
        max_offset_value = event_list.max_event_offset(source_event_list)

        if event_label_list is None:
            event_label_list = event_list.unique_event_labels(source_event_list)

    else:
        raise ValueError('Unknown source_event_list type.')

    # Initialize event roll
    event_roll = numpy.zeros((int(math.ceil(max_offset_value * 1 / time_resolution)), len(event_label_list)))

    # Fill-in event_roll
    for event in source_event_list:
        pos = event_label_list.index(event['event_label'])

        if 'event_onset' in event and 'event_offset' in event:
            event_onset = event['event_onset']
            event_offset = event['event_offset']

        elif 'onset' in event and 'offset' in event:
            event_onset = event['onset']
            event_offset = event['offset']

        onset = int(math.floor(event_onset * 1 / float(time_resolution)))
        offset = int(math.ceil(event_offset * 1 / float(time_resolution)))

        event_roll[onset:offset, pos] = 1

    return event_roll


def pad_event_roll(event_roll, length):
    """Pad event roll's length to given length

    Parameters
    ----------
    event_roll: np.ndarray, shape=(m,k)
        Event roll

    length : int
        Length to be padded

    Returns
    -------
    event_roll: np.ndarray, shape=(m,k)
        Padded event roll

    """

    if length > event_roll.shape[0]:
        padding = numpy.zeros((length-event_roll.shape[0], event_roll.shape[1]))
        event_roll = numpy.vstack((event_roll, padding))

    return event_roll


def match_event_roll_lengths(event_roll_a, event_roll_b, length=None):
    """Fix the length of two event rolls

    Parameters
    ----------
    event_roll_a: np.ndarray, shape=(m1,k)
        Event roll A

    event_roll_b: np.ndarray, shape=(m2,k)
        Event roll B

    length: int, optional
        Length of the event roll, if none given, shorter event roll is padded to match longer one.

    Returns
    -------
    event_roll_a: np.ndarray, shape=(max(m1,m2),k)
        Padded event roll A

    event_roll_b: np.ndarray, shape=(max(m1,m2),k)
        Padded event roll B

    """

    # Fix durations of both event_rolls to be equal
    if length is None:
        length = max(event_roll_b.shape[0], event_roll_a.shape[0])

    else:
        length = int(length)

    if length < event_roll_a.shape[0]:
        event_roll_a = event_roll_a[0:length, :]

    else:
        event_roll_a = pad_event_roll(
            event_roll=event_roll_a,
            length=length
        )

    if length < event_roll_b.shape[0]:
        event_roll_b = event_roll_b[0:length, :]

    else:
        event_roll_b = pad_event_roll(
            event_roll=event_roll_b,
            length=length
        )

    return event_roll_a, event_roll_b




#!/usr/bin/env python
# -*- encoding: utf-8 -*-

"""
@File        :   eval_nightly_sleep_metrics 
@Time        :   2023/9/27 14:30
@Author      :   Xuesong Chen
@Description :   
"""

from wuji.Evaluator.NightlySleepMetrics import NightlySleepMetricsEvaluator

if __name__ == '__main__':
    gt = [1, 2, 3, 4, 5]
    pred = [1, 2, 4, 4, 5]
    evaluator = NightlySleepMetricsEvaluator(gt, pred)
    # print(evaluator.pearsonr(plot=True))
    print(evaluator.pearsonr())
    print(evaluator.r2_score())
    evaluator.bland_altman_plot()
    print(evaluator.screen_OSA(3.5))
    pass


#!/usr/bin/env python
# -*- encoding: utf-8 -*-

"""
@File        :   __init__.py 
@Time        :   2023/11/21 14:06
@Author      :   Xuesong Chen
@Description :   
"""

from .signal import interp_signal


#!/usr/bin/env python
# -*- encoding: utf-8 -*-

"""
@File        :   __init__.py 
@Time        :   2023/8/17 16:45
@Author      :   Xuesong Chen
@Description :   
"""

from .Annotation.NSRR import NSRRAnnotationReader
from .Annotation.Philips import PhilipsAnnotationReader

from .Annotation.Base import AHI

from .EDF.NSRR import NSRREDFReader
from .EDF.Philips import PhilipsEDFReader




#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Scene list handling
"""

import dcase_util

__all__ = ['unique_scene_labels']


def unique_scene_labels(scene_list):
    """Find the unique scene labels

    Parameters
    ----------
    scene_list : list, shape=(n,)
        A list containing scene dicts

    Returns
    -------
    labels: list, shape=(n,)
        Unique labels in alphabetical order

    """
    if isinstance(scene_list, dcase_util.containers.MetaDataContainer):
        return scene_list.unique_scene_labels

    else:
        labels = []
        for item in scene_list:
            if 'scene_label' in item and item['scene_label'] not in labels:
                labels.append(item['scene_label'])

        labels.sort()
        return labels

